#!/bin/bash
#

# build_geant4.sh <product_directory> <gcc47|e2|e4> <debug|opt|prof> [tar] [source]

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <gcc47|e2|e4> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

select_gcc()
{
    if [ "${basequal}" = "e2" ] ||  [ "${basequal}" = "gcc47" ]
    then
       gccdotver=4.7
       gccver=v4_7_1
    elif [ "${basequal}" = "e4" ] ||  [ "${basequal}" = "gcc48" ]
    then
       gccdotver=4.8
       gccver=v4_8_1
    fi
}

check_gcc()
{
    gccshortver=`gcc --version | head -1 | cut -f3 -d" " | cut -f1-2 -d"."`
    gccfullver=`gcc --version | head -1 | cut -f3 -d" "`
    if [ "${gccshortver}" = "${gccdotver}" ]
    then
      echo " OK - gcc version is ${gccfullver}"
    else
      echo "ERROR: wrong gcc version ${gccfullver}"
      exit 1
    fi
}

setup_gcc()
{
    setup gcc ${gccver}
    check_gcc
}

declare_product()
{
    echo "Declaring ${package} ${pkgver} for ${flvr} and ${fullqual}"
    if [ ${OS1} = "Darwin" ]
    then
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -2 -m ${package}.table -q ${fullqual} -z ${product_dir}
    else 
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -4 -m ${package}.table -q ${fullqual} -z ${product_dir}
    fi
}

make_distribution()
{
    if [ "${maketar}" = "tar" ] 
    then
       if [ -d ${pkgdir}/lib64 ] || [ -d ${pkgdir}/lib ]
       then
	   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
	   exit 0
       fi
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
sourcetar=${5}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=geant4
pkgver=v4_9_6_p02
ssibuildshims_version=v0_06_07
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgbasever=`echo ${pkgdotver} | cut -f 2-4 -d "." | sed -e 's/p0//'`
pkgtarfile=geant${pkgdotver}.tar.gz

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
  ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

set_qualifiers

get_platform_name

select_gcc

make_distribution

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

declare_product

set -x

cd ${sourcedir}
tar xzf ${tardir}/${pkgtarfile}

ups build geant4 ${pkgver} -q ${fullqual} -z ${product_dir}

set +x

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}

if [ ! -d ${G4LIB} ]
then
   echo "ERROR: failed to create ${G4LIB}"
   echo "       Something is very wrong"
   exit 1
fi
echo "G4LIB is ${G4LIB}"

# remove build directory from prof and opt distributions
if [ "${extraqual}" = "opt" ]
then
  if [ -d ${pkgdir}/build ]
  then
    rm -rf ${pkgdir}/build
  fi
elif [ "${extraqual}" = "prof" ]
then
  if [ -d ${pkgdir}/build ]
  then
    rm -rf ${pkgdir}/build
  fi
fi

# need to fix paths:
# bin/geant4-config
# lib64/Geant4-xxx/Geant4*.cmake
# share/Geant4-xxx/geant4make/geant4make*sh

fixfile=/tmp/g4fix$$$$.sh
rm -f $fixfile

echo "#!/bin/bash" > $fixfile
##echo " set -x " >>  $fixfile
echo "mv ${GEANT4_FQ_DIR}/bin/geant4-config ${GEANT4_FQ_DIR}/bin/geant4-config.bak" >> $fixfile
echo "cat ${GEANT4_FQ_DIR}/bin/geant4-config.bak | sed 's%${CLHEP_BASE_DIR}%\${CLHEP_BASE_DIR}%'  | sed 's%${XERCESCROOT}%\${XERCESCROOT}%' > ${GEANT4_FQ_DIR}/bin/geant4-config" >> $fixfile
echo "chmod +x  ${GEANT4_FQ_DIR}/bin/geant4-config" >> $fixfile

cmakeConfig=`ls ${G4LIB}/Geant4-${pkgbasever}/Geant4*.cmake`
for file in ${cmakeConfig}
do
  echo "mv ${file} ${file}.bak" >> $fixfile
  echo "cat ${file}.bak | sed 's%${CLHEP_BASE_DIR}%\$ENV{CLHEP_BASE_DIR}%'  | sed 's%${XERCESCROOT}%\$ENV{XERCESCROOT}%' > ${file}" >> $fixfile
done

g4make=`ls ${GEANT4_FQ_DIR}/share/Geant4-${pkgbasever}/geant4make/geant4make*sh`
for file in ${g4make}
do
  echo "mv ${file} ${file}.bak" >> $fixfile
  echo "cat ${file}.bak | sed 's%${CLHEP_BASE_DIR}%\${CLHEP_BASE_DIR}%'  | sed 's%${XERCESCROOT}%\${XERCESCROOT}%' > ${file}" >> $fixfile
  echo "chmod +x  ${file}" >> $fixfile
done

chmod +x $fixfile
$fixfile

rm -f $fixfile

# this must be last
make_distribution

exit 0
